#include "ScriptingDock.hxx"

namespace {
	SCM handler (void*, SCM tag, SCM throw_args) {
		scm_handle_by_message_noexit ((void*)"foo", tag, throw_args);
		return SCM_BOOL_F;
	}
}

ScriptingDock::ScriptingDock(const QString& title, QWidget * parent)
	: QDockWidget(title, parent)
	, logger(log4cxx::Logger::getLogger("ScriptingDock")) {
	QTabWidget * tab = new QTabWidget;
	QWidget * widget = new QWidget;
	widget->setLayout(layout = new QGridLayout);
	layout->addWidget(browser = new QTextBrowser, 0, 0, 1, 0);
	layout->addWidget(line = new QLineEdit, 1, 0);
	layout->addWidget(button = new QPushButton(tr("Evaluate")), 1, 1);
	tab->addTab(widget, "GUILE");
	((QDockWidget*)this)->setWidget(tab);
	connect(button, SIGNAL(released()), this, SLOT(doEvaluate()));
	connect(line, SIGNAL(returnPressed()), this, SLOT(doEvaluate()));
	scm_init_guile();
	scm_internal_catch(SCM_BOOL_T,
	                   (SCM (*)(void *))scm_c_eval_string,
	                   (void*)"(use-modules (system repl server))",
	                   handler, NULL);
	scm_internal_catch(SCM_BOOL_T,
	                   (SCM (*)(void *))scm_c_eval_string,
	                   (void*)"(spawn-server)",
	                   handler, NULL);

	guile_output_port = scm_open_output_string();
	guile_error_port = scm_open_output_string();
	scm_set_current_output_port(guile_output_port);
	scm_set_current_error_port(guile_error_port);
}


void ScriptingDock::doEvaluate() {
	QString text = line->text();
	line->clear();
	LOG4CXX_INFO(logger, "Evaluating String \"" << text.toStdString() << "\"");
	browser->append(QString("> ") + text);

	SCM result_obj = scm_internal_catch(SCM_BOOL_T,
	                                    (SCM (*)(void *))scm_c_eval_string,
	                                    (void*)text.toStdString().c_str(),
	                                    handler, NULL);
	SCM result_str = scm_object_to_string(result_obj, SCM_UNDEFINED);

	SCM output = scm_get_output_string(guile_output_port);
	QString output_q = scm_to_locale_string(output);
	if (output_q.endsWith("\n")) output_q.chop(1);
	if (output_q != "") browser->append(output_q);

	output =  scm_get_output_string(guile_error_port);
	output_q = scm_to_locale_string(output);
	if (output_q.endsWith("\n")) output_q.chop(1);
	if (output_q != "") browser->append(output_q);

	scm_truncate_file(guile_output_port, scm_from_uint16(0));
	scm_truncate_file(guile_error_port, scm_from_uint16(0));

	browser->append(scm_to_locale_string(result_str));
}
